@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,14 +24,23 @@
 #define ZBX_STR(str)	ZBX_STR2(str)
 
 #define APPLICATION_NAME	"Zabbix Agent"
-#define ZABBIX_REVDATE		"21 December 2016"
-#define ZABBIX_VERSION_MAJOR	3
+#define ZABBIX_REVDATE		"8 March 2023"
+#define ZABBIX_VERSION_MAJOR	6
 #define ZABBIX_VERSION_MINOR	0
-#define ZABBIX_VERSION_PATCH	7
-#define ZABBIX_VERSION_REVISION	64609
+#define ZABBIX_VERSION_PATCH	14
+#ifndef ZABBIX_VERSION_REVISION
+#	define ZABBIX_VERSION_REVISION	3f184b456c7
+#endif
+#ifdef _WINDOWS
+#	ifndef ZABBIX_VERSION_RC_NUM
+#		define ZABBIX_VERSION_RC_NUM	1400
+#	endif
+#endif
 #define ZABBIX_VERSION_RC	""
 #define ZABBIX_VERSION		ZBX_STR(ZABBIX_VERSION_MAJOR) "." ZBX_STR(ZABBIX_VERSION_MINOR) "." \
 				ZBX_STR(ZABBIX_VERSION_PATCH) ZABBIX_VERSION_RC
 #define ZABBIX_REVISION		ZBX_STR(ZABBIX_VERSION_REVISION)
 
+int	zbx_get_component_version(char *value);
+
 #endif
