@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 #include "sysinfo.h"
 #include "zbxjson.h"
 #include "log.h"
+#include "zbxalgo.h"
+#include "inodes.h"
 
 static int	get_fs_size_stat(const char *fs, zbx_uint64_t *total, zbx_uint64_t *free,
 		zbx_uint64_t *used, double *pfree, double *pused, char **error)
@@ -37,6 +39,7 @@ static int	get_fs_size_stat(const char *fs, zbx_uint64_t *total, zbx_uint64_t *f
 	if (0 != ZBX_STATFS(fs, &s))
 	{
 		*error = zbx_dsprintf(NULL, "Cannot obtain filesystem information: %s", zbx_strerror(errno));
+		zabbix_log(LOG_LEVEL_DEBUG,"%s failed with error: %s",__func__, *error);
 		return SYSINFO_RET_FAIL;
 	}
 
@@ -199,21 +202,19 @@ int	VFS_FS_DISCOVERY(AGENT_REQUEST *request, AGENT_RESULT *result)
 	struct statvfs	*mntbuf;
 	struct zbx_json	j;
 
-	if (0 == (rc = getmntinfo(&mntbuf, MNT_WAIT)))
+	if (0 == (rc = getmntinfo(&mntbuf, MNT_NOWAIT)))
 	{
 		SET_MSG_RESULT(result, zbx_dsprintf(NULL, "Cannot obtain system information: %s", zbx_strerror(errno)));
 		return SYSINFO_RET_FAIL;
 	}
 
-	zbx_json_init(&j, ZBX_JSON_STAT_BUF_LEN);
-
-	zbx_json_addarray(&j, ZBX_PROTO_TAG_DATA);
+	zbx_json_initarray(&j, ZBX_JSON_STAT_BUF_LEN);
 
 	for (i = 0; i < rc; i++)
 	{
 		zbx_json_addobject(&j, NULL);
-		zbx_json_addstring(&j, "{#FSNAME}", mntbuf[i].f_mntonname, ZBX_JSON_TYPE_STRING);
-		zbx_json_addstring(&j, "{#FSTYPE}", mntbuf[i].f_fstypename, ZBX_JSON_TYPE_STRING);
+		zbx_json_addstring(&j, ZBX_LLD_MACRO_FSNAME, mntbuf[i].f_mntonname, ZBX_JSON_TYPE_STRING);
+		zbx_json_addstring(&j, ZBX_LLD_MACRO_FSTYPE, mntbuf[i].f_fstypename, ZBX_JSON_TYPE_STRING);
 		zbx_json_close(&j);
 	}
 
@@ -225,3 +226,114 @@ int	VFS_FS_DISCOVERY(AGENT_REQUEST *request, AGENT_RESULT *result)
 
 	return SYSINFO_RET_OK;
 }
+
+static int	vfs_fs_get(AGENT_REQUEST *request, AGENT_RESULT *result)
+{
+	int			i, rc;
+	struct statvfs		*mntbuf;
+	struct zbx_json		j;
+	zbx_uint64_t		total, not_used, used;
+	zbx_uint64_t		itotal, inot_used, iused;
+	double			pfree, pused;
+	double			ipfree, ipused;
+	char 			*error;
+	zbx_vector_ptr_t	mntpoints;
+	zbx_mpoint_t		*mntpoint;
+	char 			*mpoint;
+	int			ret = SYSINFO_RET_FAIL;
+
+	/* check how many bytes to allocate for the mounted filesystems */
+	if (0 == (rc = getmntinfo(&mntbuf, MNT_NOWAIT)))
+	{
+		SET_MSG_RESULT(result, zbx_dsprintf(NULL, "Cannot obtain system information: %s", zbx_strerror(errno)));
+		return SYSINFO_RET_FAIL;
+	}
+
+	zbx_vector_ptr_create(&mntpoints);
+
+	for (i = 0; i < rc; i++)
+	{
+		mpoint = mntbuf[i].f_mntonname;
+		if (SYSINFO_RET_OK != get_fs_size_stat(mpoint, &total, &not_used, &used, &pfree, &pused,&error))
+		{
+			zbx_free(error);
+			continue;
+		}
+		if (SYSINFO_RET_OK != get_fs_inode_stat(mpoint, &itotal, &inot_used, &iused, &ipfree, &ipused, "pused",
+				&error))
+		{
+			zbx_free(error);
+			continue;
+		}
+
+		mntpoint = (zbx_mpoint_t *)zbx_malloc(NULL, sizeof(zbx_mpoint_t));
+		zbx_strlcpy(mntpoint->fsname, mpoint, MAX_STRING_LEN);
+		zbx_strlcpy(mntpoint->fstype, mntbuf[i].f_fstypename, MAX_STRING_LEN);
+		mntpoint->bytes.total = total;
+		mntpoint->bytes.used = used;
+		mntpoint->bytes.not_used = not_used;
+		mntpoint->bytes.pfree = pfree;
+		mntpoint->bytes.pused = pused;
+		mntpoint->inodes.total = itotal;
+		mntpoint->inodes.used = iused;
+		mntpoint->inodes.not_used = inot_used;
+		mntpoint->inodes.pfree = ipfree;
+		mntpoint->inodes.pused = ipused;
+
+		zbx_vector_ptr_append(&mntpoints, mntpoint);
+	}
+
+	if (0 == (rc = getmntinfo(&mntbuf, MNT_NOWAIT)))
+	{
+		SET_MSG_RESULT(result, zbx_dsprintf(NULL, "Cannot obtain system information: %s", zbx_strerror(errno)));
+		goto out;
+	}
+	zbx_json_initarray(&j, ZBX_JSON_STAT_BUF_LEN);
+
+	for (i = 0; i < rc; i++)
+	{
+		int idx;
+		mpoint = mntbuf[i].f_mntonname;
+
+		if (FAIL != (idx = zbx_vector_ptr_search(&mntpoints, mpoint, ZBX_DEFAULT_STR_COMPARE_FUNC)))
+		{
+			mntpoint = (zbx_mpoint_t *)mntpoints.values[idx];
+			zbx_json_addobject(&j, NULL);
+			zbx_json_addstring(&j, ZBX_SYSINFO_TAG_FSNAME, mntpoint->fsname, ZBX_JSON_TYPE_STRING);
+			zbx_json_addstring(&j, ZBX_SYSINFO_TAG_FSTYPE, mntpoint->fstype, ZBX_JSON_TYPE_STRING);
+			zbx_json_addobject(&j, ZBX_SYSINFO_TAG_BYTES);
+			zbx_json_adduint64(&j, ZBX_SYSINFO_TAG_TOTAL, mntpoint->bytes.total);
+			zbx_json_adduint64(&j, ZBX_SYSINFO_TAG_FREE, mntpoint->bytes.not_used);
+			zbx_json_adduint64(&j, ZBX_SYSINFO_TAG_USED, mntpoint->bytes.used);
+			zbx_json_addfloat(&j, ZBX_SYSINFO_TAG_PFREE, mntpoint->bytes.pfree);
+			zbx_json_addfloat(&j, ZBX_SYSINFO_TAG_PUSED, mntpoint->bytes.pused);
+			zbx_json_close(&j);
+			zbx_json_addobject(&j, ZBX_SYSINFO_TAG_INODES);
+			zbx_json_adduint64(&j, ZBX_SYSINFO_TAG_TOTAL, mntpoint->inodes.total);
+			zbx_json_adduint64(&j, ZBX_SYSINFO_TAG_FREE, mntpoint->inodes.not_used);
+			zbx_json_adduint64(&j, ZBX_SYSINFO_TAG_USED, mntpoint->inodes.used);
+			zbx_json_addfloat(&j, ZBX_SYSINFO_TAG_PFREE, mntpoint->inodes.pfree);
+			zbx_json_addfloat(&j, ZBX_SYSINFO_TAG_PUSED, mntpoint->inodes.pused);
+			zbx_json_close(&j);
+			zbx_json_close(&j);
+		}
+	}
+
+	zbx_json_close(&j);
+
+	SET_STR_RESULT(result, zbx_strdup(NULL, j.buffer));
+
+	zbx_json_free(&j);
+	ret = SYSINFO_RET_OK;
+out:
+	zbx_vector_ptr_clear_ext(&mntpoints, (zbx_clean_func_t)zbx_mpoints_free);
+	zbx_vector_ptr_destroy(&mntpoints);
+
+	return ret;
+}
+
+int	VFS_FS_GET(AGENT_REQUEST *request, AGENT_RESULT *result)
+{
+	return zbx_execute_threaded_metric(vfs_fs_get, request, result);
+
+}
